#!/usr/bin/env bash

set -e

unset LD_{RUN,LIBRARY}_PATH LIBRARY_PATH CPATH {C,CPLUS}_INCLUDE_PATH PYTHONPATH
unset "${!EUPS_@}" "${!LSST_@}" SETUP_EUPS

pushd "$(dirname -- "$0")"
  scriptdir="$PWD"
popd

source "$scriptdir/lib/install.rc"
source "$scriptdir/libversions"
source "$scriptdir/variables"

if [[ $nproc -ge 2 ]]
then
  export MAKEFLAGS="-j $(($nproc))"
  export EUPSPKG_NJOBS="$(($nproc))"
  # export EUPSPKG_SCONSFLAGS="-j $(($nproc)) opt=3"
fi

workdir="${workdir:-/tmp}"/hscpipe_build_dir

lsst_home="${prefix}/lsst_home"


main()
(
  export PATH="$prefix/bin:$path"

  install_gcc
  install_binutils
  install_git
  install_sqlite3
  install_python

  install_openblas

  if [[ "$minimal_python_lib" = "False" ]]
  then
    # These libraries are not essential.
    # They are not necessary during the build process,
    # nor will users use them in a usual scenario.
    pip3 install \
      psycopg2-binary \
      ;
  fi

  # We need to pre-install several python libraries.
  # To know what to pre-install:
  #     'find {stack_dir}/Linux64 -name check_python_module'
  # and https://github.com/HyperSuprime-Cam/scipipe_conda_env/blob/master/etc/conda3_packages-linux-64.yml
  pip3 install \
    numpy \
    scipy \
    matplotlib \
    astropy \
    ;
  # Some libraries must be of specific versions,
  # which must be installed last of all
  # because it is highly possible that they would be replaced by newer versions
  # on further calls to pip3.
  pip3 install \
    setuptools==47.3.1 \
    ;
  export EUPS_PKGROOT_BASE_URL="${eups_pkgroot_base_url}"

  # install source patches
  find "$scriptdir/patches" -maxdepth 1 -name "*.patch" -print0 | while read -d '' patch
  do
    package="$(basename "$patch")"
    package="${package%.*}"
    if [[ ! -e "$workdir/eupsrepo/products/$package.eupspkg" ]]
    then
      bash "$scriptdir/clone_repo.sh" "$package"
      pushd "$workdir/eupsrepo/products"
        tar -xzf "$package.eupspkg"
        patch -p0 < "$patch"
        tar -czf - "$package" > "$package.eupspkg"
      popd
    fi
  done

  mkdir -p "${lsst_home}"
  pushd "${lsst_home}"

  install_eups

  # We replace the build directory (EupsBuildDir)
  # with a symbolic link to a subdirectory of $workdir
  if :
  then
    miniconda_dir=(stack/miniconda*)
    rm -rf "$miniconda_dir/EupsBuildDir"
    mkdir -p "$workdir/EupsBuildDir/Linux64"
    mkdir "$miniconda_dir/EupsBuildDir"
    ln -s "$workdir/EupsBuildDir/Linux64" "$miniconda_dir/EupsBuildDir"
  fi

  popd

  cleaner_thread "${lsst_home}/${miniconda_dir}" &

  source "${lsst_home}"/loadLSST.bash
  (
    env EUPS_PKGROOT="$workdir/eupsrepo|${EUPS_PKGROOT_BASE_URL%/}/" \
      eups distrib install hscPipe "${hscpipe_ver}"
  )

  rm -rf "${lsst_home}/${miniconda_dir}/EupsBuildDir"

  strip_hack_from_gcc

  # install bashrc
  cat "$(dirname "$0")"/rsrc/bashrc.in |
    sed s$'\xff'@prefix@$'\xff'"$prefix"$'\xff'g |
    sed s$'\xff'@lsst_home@$'\xff'"$LSST_HOME"$'\xff'g |
    sed s$'\xff'@hscpipe_ver@$'\xff'"$hscpipe_ver"$'\xff'g \
    > "$prefix"/bashrc

  # setup-eups() in bashrc needs $prefix/bin/check-requisites-hscPipe-${hscpipe_ver},
  # which will be created by post_build.sh. For users who won't run
  # post_build.sh, we place an empty exe there

  echo "int main(){ return 0; }" > "$workdir"/check-requisites.c
  gcc -O3 "$workdir"/check-requisites.c \
    -o "${prefix}/bin/check-requisites-hscPipe-${hscpipe_ver}"

  # Place create-rootdir.sh in bin/
  cat "$(dirname "$0")"/rsrc/create-rootdir.sh > "$prefix"/bin/create-rootdir.sh
  chmod a+x "$prefix"/bin/create-rootdir.sh

  # remove ".tests" directories
  find "${prefix}" -type d -name ".tests" -exec rm -rf {} +

  # remove build.log
  find "${prefix}" -path "*/ups/build.log" -delete
)

cleaner_thread()
(
  cd "$1"
  while [[ -e EupsBuildDir ]]
  do
    sleep 60
    dirs=($(ls -d -t -1 EupsBuildDir/Linux64/* 2> /dev/null))
    # remove all the dirs except the newest ones
    for((i = 3; i < ${#dirs[@]}; ++i))
    do
      rm -rf "${dirs[i]}"
    done
  done

  echo "EupsBuildDir cleaner died. (This is not an error)"
)

main
